;******************************************************************************
;   MSP430F413 & TMP100 Software Interface
; 
;   Description: I2C communication with a TMP100 in one-shot conversion mode 
;   is demonstrated. Temperature is read from the TMP100 and displayed to LCD.
;   Only upper 8-bits from TMP100 temperature register is read, representing 
;   7Fh = 128oC to 80h = -128oC in 1oC steps. 
;   MSP430 is I2C master with TMP100 as the slave. Ack error checking on WR
;   not implemented. DCO/FLL generated MCLK = 1MHz. LMP3 implemented with
;   wake-up every 2 seconds as defined by Basic Timer interrupt from
;   watch crystal.
;
;          /|\            /|\ /|\
;           |    TMP100   10k 10k  MSP430F413           
;           |    -------   |   |  ---------- 
;           +---|Vcc SDA|<-|---+>|P2.0      |    +----------------------+
;            +--|A0     |  |     |    S0-S23|--->| Varitronix 3.5 digit |
;            |--|A1     |  |     |      COM0|--->|  Static LCD Display  |
;            |--|Vss SCL|<-+-----|P2.1      |    +----------------------+                        
;           \|/  -------         |          |
;                                |       XIN|<---32.768KHz Watch Crystal     
;
;
;
;   Z. Albus
;   Texas Instruments, Inc
;   August 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************

#include  <signal.h>
#include  <io.h>

;            CPU registers used
#define      DIGITS   R6                    
#define      RXTXI2C  R7                    
#define      ADDRI2C  R8                    
#define      DATAI2C  R9                    
#define      BITI2C   R10                   
;
;            Definitions for I2C bus
            .equ   TMPADDR,0x90             ; TMP100 HW Address (A0=A1=0+WR)
            .equ   SDA,0x01                 ; P2.0 controls SDA line (pull-up)
            .equ   SCL,0x02                 ; P2.1 controls SCL line (pull-up)
;------------------------------------------------------------------------------ 
; word & byte variables definition
            .section    .data
Neg_Sign:   .byte   0                       ; Negative Temperature Flag
                                            ;
            .section    .text
RESET:      mov.w   #__stack,R1             ; Initialize stack-pointer
            call    #Init_Sys               ; Initialize system
                                            ;
One_Shot_1: mov.b   #0x01,ADDRI2C           ; ADDRI2C = TMP100 Pointer = Config    
            mov.b   #0x81,DATAI2C           ; One-Shot Instruction Data
            call    #Write_I2C              ; Write to TMP100
                                            ;             
Mainloop:   bis.w   #LPM3,R2                ; Enter LPM3
                                            ;
Read_Temp:  mov.b   #0x00,ADDRI2C           ; ADDRI2C = TMP100 Pointer = Data
            call    #Read_I2C               ; Get Upper 8-bit Conversion Result
            and.w   #0x00FF,DATAI2C         ; Clear Upper Byte
                                            ;
            call    #Calc_Temp              ; Handle "-" Temperature Data
            call    #Disp_Signed_3_5        ; Update LCD with BCD Data
                                            ;
One_Shot:   mov.b   #0x01,ADDRI2C           ; ADDRI2C = TMP100 Pointer = Config
            mov.b   #0x81,DATAI2C           ; One-Shot Instruction Data
            call    #Write_I2C              ; Write to TMP100
                                            ;
Main_End:   jmp     Mainloop                ; Loop Again
                                            ;
;------------------------------------------------------------------------------ 
Init_Sys:   ;Subroutine sets up  Modules and Control Registers
;------------------------------------------------------------------------------ 
            mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop WDT
SetupFLL2:  bis.b   #XCAP18PF,&FLL_CTL0     ; Set load capacitance for xtal
SetupLCD:   mov.b   #0xC5,&LCDCTL           ; Static LCD, segments = 0 - 23
SetupBT:    mov.b   #BTFRFQ1+BTFRFQ0+BTDIV+BTIP2+BTIP1+BTIP0,&BTCTL;
                                            ; 2s BT Int, Set LCD freq
            bis.b   #BTIE,&IE2              ; Enable Basic Timer interrupt      
SetupPorts: mov.b   #0xFF,&P1DIR            ; Set port to outputs
            clr.b   &P1OUT                  ; P1OUTs = 0
            mov.b   #0xFF,&P2DIR            ; Set port to outputs
            clr.b   &P2OUT                  ; P2OUTs = 0
            mov.b   #0xFF,&P3DIR            ; Set port to outputs
            clr.b   &P3OUT                  ; P3OUTs = 0
            mov.b   #0xFF,&P4DIR            ; Set port to outputs
            clr.b   &P4OUT                  ; P4OUTs = 0
            mov.b   #0xFF,&P5DIR            ; Set port to outputs
            clr.b   &P5OUT                  ; P5OUTs = 0
            mov.b   #0xFF,&P6DIR            ; Set port to outputs
            clr.b   &P6OUT                  ; P6OUTs = 0
                                            ;
            eint                            ; Enable Interrupts
                                            ;
            ret                             ; Return to Main
                                            ;            
;///////////I2C Subroutines start////////////////////////////////////////////// 
;------------------------------------------------------------------------------ 
Read_I2C:  ; Reads two bytes of data transmitted from slave
;            enter ADDRI2C=00 - FF I2C device address to read
;                  RXTXI2C=x
;                  DATAI2C=x
;            exit  ADDRI2C=x
;                  RXTXI2C=x
;                  DATAI2C=0000 - FFFF I2C device data
;------------------------------------------------------------------------------ 
            mov.b   #TMPADDR,RXTXI2C        ; Load HW Address
            call    #I2C_Start              ; Send Start, Address and Ack
            mov.b   ADDRI2C,RXTXI2C         ; Load Pointer Address
            call    #I2C_TX                 ; Send Pointer and Ack
            mov.b   #TMPADDR,RXTXI2C        ; Load HW Address
            bis.b   #0x1,RXTXI2C            ; Set LSB for "READ"
            call    #I2C_Start              ; Send Start, Address+RD and Ack
            call    #I2C_RX                 ; Read Data and Ack
                                            ;
;***** Used for 2-Byte transfer only *****  ;
;            call    #I2C_ACKn               ; Acknowledge Byte Rcv'd
;            call    #I2C_RX                 ; Read Data and Ack
                                            ;
            call    #I2C_NACKn              ; NOT Acknowledge Byte Rcv'd
            call    #I2C_Stop               ; Send Stop
            ret                             ; Return from subroutine
                                            ;
;------------------------------------------------------------------------------ 
Write_I2C:  ;enter ADDRI2C=00 - FF I2C device address to write to
;                 RXTXI2C=x
;                 DATAI2C=00 - FF I2C device data to write
;           exit  ADDRI2C=x
;                 RXTXI2C=x
;                 DATAI2C=x
;------------------------------------------------------------------------------ 
            mov.b   #TMPADDR,RXTXI2C        ; Load HW Address
            call    #I2C_Start              ; Send Start, Address and Ack
            mov.b   ADDRI2C,RXTXI2C         ; Load Pointer Address
            call    #I2C_TX                 ; Send Pointer and Ack
            mov.b   DATAI2C,RXTXI2C         ; Load Out-Going Data
            call    #I2C_TX                 ; Send Data and Ack
            call    #I2C_Stop               ; Send Stop
            ret                             ; Return from subroutine
                                            ;
;------------------------------------------------------------------------------ 
I2C_Start:  ;enter SDA=1, SCL=x
;           exit  SDA=1, SCL=0
;------------------------------------------------------------------------------ 
            bic.b   #SCL+SDA,&P2DIR         ; SCL and SDA to input direction
            bic.b   #SCL+SDA,&P2OUT         ; SCL=1, SDA=1
            bis.b   #SDA,&P2DIR             ; SDA=0
            bis.b   #SCL,&P2DIR             ; SCL=0
;------------------------------------------------------------------------------ 
I2C_TX:     ;enter SDA=x, SCL=0
;           exit  SDA=1, SCL=0
;------------------------------------------------------------------------------ 
            mov     #8,BITI2C              ; number of bits to xfer
I2C_TX_Bit: rla.b   RXTXI2C                 ; data bit -> carry
            jc      I2C_TX1                 ; test carry for 1 or 0
I2C_TX0:    bis.b   #SDA,&P2DIR             ; SDA=0
            jmp     I2C_TXx                 ; Toggle SCL
I2C_TX1:    bic.b   #SDA,&P2DIR             ; SDA=1
I2C_TXx:    bic.b   #SCL,&P2DIR             ; SCL=1
            nop                             ; delay to meet I2C spec
            nop                             ;
            bis.b   #SCL,&P2DIR             ; SCL=0
            dec     BITI2C                  ; all bits read?
            jnz     I2C_TX_Bit              ; continue until 8 bits are sent
            bic.b   #SDA,&P2DIR             ; SDA=1
                                            ;
TX_Ackn:    bic.b   #SCL,&P2DIR             ; SCL=1
            nop                             ; delay to meet I2C spec
            nop                             ;
            bis.b   #SCL,&P2DIR             ; SCL=0
                                            ;
            ret                             ; Return from subroutine
                                            ;
;------------------------------------------------------------------------------ 
I2C_RX:     ;enter SDA=1, SCL=0
;           exit  SDA=x, SCL=0
;------------------------------------------------------------------------------ 
            mov.b   #08,BITI2C              ; number of bits to rcv
I2C_RX_Bit: bic.b   #SCL,&P2DIR             ; SCL=1
            bit.b   #SDA,&P2IN              ; SDA bit -> carry
            rlc.w   DATAI2C                 ; Shift new bit into DATAI2C
            bis.b   #SCL,&P2DIR             ; SCL=0
            dec     BITI2C                  ; all bits read?
            jnz     I2C_RX_Bit              ; continue until 8 bits are read
                                            ;
            ret                             ; Return from subroutine
                                            ;
                                            ;
;------------------------------------------------------------------------------ 
I2C_ACKn:   ;enter SDA=x, SCL=0
;           exit  SDA=0, SCL=0
;------------------------------------------------------------------------------ 
            bis.b   #SDA,&P2DIR             ; SDA=0, Ack
            bic.b   #SCL,&P2DIR             ; SCL=1
            bis.b   #SCL,&P2DIR             ; SCL=0
            bic.b   #SDA,&P2DIR             ; SDA=1
Ackn_End:   ret                             ; Return from subroutine
                                            ;
;------------------------------------------------------------------------------ 
I2C_NACKn:  ;enter SDA=x, SCL=0
;           exit  SDA=1, SCL=0
;------------------------------------------------------------------------------ 
            bic.b   #SDA,&P2DIR             ; SDA=1, NOT Ack
            bic.b   #SCL,&P2DIR             ; SCL=1
            bis.b   #SCL,&P2DIR             ; SCL=0
NAckn_End:  ret                             ; Return from subroutine
                                            ;
;------------------------------------------------------------------------------ 
I2C_Stop:   ;enter SDA=x, SCL=0
;           exit  SDA=1, SCL=1
;------------------------------------------------------------------------------ 
            bis.b   #SDA,&P2DIR             ; SDA = 0
            bic.b   #SCL,&P2DIR             ; SCL = 1
            bic.b   #SDA,&P2DIR             ; SDA = 1
I2C_End:    ret                             ; Return from subroutine
                                            ;
;///////////I2C Subroutines stop///////////////////////////////////////////////

;------------------------------------------------------------------------------ 
Calc_Temp: ;Subroutine handles MSB decoding for negative temperatures
           ;CPU Registers used R13, not saved
;------------------------------------------------------------------------------ 
            bic.b   #0x01,Neg_Sign          ; Clear negative Flag
            bit.b   #0x80,DATAI2C           ; Test for new negative result
            jz      Calc_End                ; Jump if number is positive   
            bis.b   #0x01,Neg_Sign          ; Set negative flag
            and.b   #0x7F,DATAI2C           ; Clear MSB (negative holder)
            mov.b   #0x80,R13               ; 128dec to R13 (128-DATAI2C=Value)
            sub.b   DATAI2C,R13             ; R13 is the result now
            mov.b   R13,DATAI2C             ; New Data to convert to BCD
                                            ;
Calc_End:   ret                             ; Return from subroutine
                                            ;
;------------------------------------------------------------------------------ 
Disp_Signed_3_5:  ;Subroutine to Display 3.5 BCD digits in DIGITS w/ sign
                  ;CPU Registers used R15, R14, R13, not saved
;------------------------------------------------------------------------------ 
Disp_1:     mov.w   DATAI2C,DIGITS          ; Move data to DIGITs
            call    #bin2bcd                ; Convert to BDC format
            rlc.w   DIGITS                  ; Shift Data 4 bits
            rlc.w   DIGITS                  ;
            rlc.w   DIGITS                  ;
            rlc.w   DIGITS                  ;
            and.w   #0xFFF0,DIGITS          ; Clear LSByte
            add.w   #0x000C,DIGITS          ; Add "C"
                                            ;
            mov.w   DIGITS,R13              ; Preserve DIGITS
            mov.w   #LCDM1,R15              ; R15 points to first LCD location
                                            ;
Disp_2:     mov.w   R13,R14                 ; LSD value moved to R14
            and.b   #0xF,R14                ; R14 = Nibble now 
            mov.b   LCD_Tab(R14),R14        ; R14 = LCD segment information
            mov.b   R14,0(R15)              ; Segments a & b to LCD
            rra.w   R14                     ;
            mov.b   R14,1(R15)              ; Segments c & d to LCD
            rra.w   R14                     ;
            mov.b   R14,2(R15)              ; Segments e & f to LCD
            rra.w   R14                     ;
            mov.b   R14,3(R15)              ; Segments g & h to LCD
                                            ;
            rra.w   R13                     ; Right Shift 
            rra.w   R13                     ; Right Shift 
            rra.w   R13                     ; Right Shift 
            rra.w   R13                     ; Right Shift 
                                            ;
            add.b   #0x4,R15                ; R15 points to next LCD location
            cmp.b   #LCDM13,R15             ; Updated all Segments?
            jne     Disp_2                  ;
                                            ;
            bit.b   #0x01,Neg_Sign          ; Display Neg Sign?
            jz      Disp_3                  ; Jump if No
            bis.b   #010h,&LCDM8            ; "-" segment active
                                            ;
Disp_3:     bit.w   #0x0001,R13             ; Data >99
            jnc     Disp_End                ;
            bis.b   #0x10,&LCDM12           ; B-C digit active
                                            ;
Disp_End:   ret                             ; Return from subroutine	
                                            ;
;-------------------------------------------------------------------------------
;             Define characters for VI-302 3.5 digit static LCD
;-------------------------------------------------------------------------------
LCD_Tab:    .byte   0x77                    ; displays "0"
            .byte   0x12                    ; displays "1"
            .byte   0x3D                    ; displays "2"
            .byte   0x3B                    ; displays "3"
            .byte   0x5A                    ; displays "4"
            .byte   0x6B                    ; displays "5"
            .byte   0x6F                    ; displays "6"
            .byte   0x13                    ; displays "7"
            .byte   0x7F                    ; displays "8"
            .byte   0x7B                    ; displays "9"
            .byte   0x5F                    ; Displays "A"
            .byte   0x00                    ; Displays  Blank
            .byte   0x65                    ; Displays "C"
            .byte   0x59                    ; Displays "degrees" o
            .byte   0x6D                    ; Displays "E"
            .byte   0x4D                    ; Displays "F"
                                            ;
;------------------------------------------------------------------------------ 
bin2bcd:    ;subroutine for converting 16-bit binary to BCD
            ;input DIGIT 16-bit binary, output DIGIT 16-bit BCD
;------------------------------------------------------------------------------ 
             mov.w   #16,R15                ;
             clr.w   R14                    ;
             clr.w   R13                    ;
L1:          rla.w   DIGITS                 ;
             dadd.w  R13,R13                ;
             dadd.w  R14,R14                ;
             dec.w   R15                    ;
             jnz     L1                     ;
             mov.w   R13,DIGITS             ;
             ret                            ; Return from subroutine
                                            ;
;---------------------------------------------------------------------------
BT_ISR:     ;Exit any LPMx Mode
;---------------------------------------------------------------------------
            mov.w   #GIE,0(R1)              ; Exit LPM3, interrupts enabled
            reti                            ; Return from interrupt		 
                                            ;   
;----------------------------------------------------------------------------- 
;           Interrupt Vectors Used MSP430F413         
;----------------------------------------------------------------------------- 
            .section    .vectors
            .org    BASICTIMER_VECTOR,0xFF  ; Basic Timer Vector
            .word   BT_ISR                  ; 
            .org    RESET_VECTOR,0xFF       ; MSP430 RESET Vector
            .word   RESET                   ; 
