#include <io.h>
;******************************************************************************
;   MSP-FET430x110 Demo - Software SPI Interface to TLV5616 12-bit DAC 
;
;   Description: Drive serial communiations with TLV5616 12-bit DAC. A sine 
;   wave is generated using a a 32-step sine look-up table Sin_Tab.  R5 is 
;   used as a pointer into Sin_Tab, R6 holds the 12-bit data and R7 is used
;   as a temporary bit counter.
;   ACLK = n/a, MCLK = SMCLK = default DCO ~ 800k 
;
;                 MSP430F1121
;             -----------------
;        /|\ |              XIN|-  
;         |  |                 |      TLV5616
;         ---|RST          XOUT|-   ----------
;            |             P2.3|-->|FS     OUT|--> ~ 75hz sine wave
;            |             P2.0|-->|DIN       |
;            |             P2.1|-->|SCLK    CS|-|
;            |                 |   |          | v
;
;
;   M.Buccini
;   Texas Instruments, Inc
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
            .equ    DOUT,0x01               ; P2.0 Data Out
            .equ    SCLK,0x02               ; P2.1 Serial Clock
            .equ    FS,0x08                 ; P2.3 Frame Sync

            .section    .text
RESET:      mov.w   #0x300,R1               ; Initialize 'x112x stackpointer
StopWDT:    mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop Watchdog Timer
SetupP1:    bis.b   #FS+SCLK,&P2OUT         ; FS, SCLK Set
            bis.b   #FS+SCLK+DOUT,&P2DIR    ; FS,Clk and DOUT as outputs
                                            ;
Mainloop:   incd.w  R5                      ; R5 = sine table pointer     
            and.w   #0x3E,R5                ; 32 steps in table             
            mov.w   Sin_tab(R5),R6          ; R6 = Sine value
                                            ;
            bic.b   #FS,&P2OUT              ; FS low, enable DAC
            mov.w   #16,R7                  ; R7 = 16 bits to shift out
DAC_Loop:   rlc.w   R6                      ; C = Data bit
Out_0:      bic.b   #DOUT,&P2OUT            ; DOUT reset
            jnc     Clk_0                   ;
Out_1:      bis.b   #DOUT,&P2OUT            ; DOUT set
Clk_0:      bic.b   #SCLK,&P2OUT            ; SCLK reset, data latched
            dec.w   R7                      ; 
Clk_1:      bis.b   #SCLK,&P2OUT            ; SCLK set 
            jnz     DAC_Loop                ;
            bis.b   #FS,&P2OUT              ; FS high, disable DAC
            jmp     Mainloop                ; Repeat
                                            ;
;------------------------------------------------------------------------------ 
; 12-bit Sine Lookup table with 32 steps
;------------------------------------------------------------------------------ 
Sin_tab:    .word    2048
            .word    2447
            .word    2831
            .word    3185
            .word    3495
            .word    3750
            .word    3939
            .word    4056
            .word    4095
            .word    4056
            .word    3939
            .word    3750
            .word    3495
            .word    3185
            .word    2831
            .word    2447
            .word    2048
            .word    1648
            .word    1264
            .word    910
            .word    600
            .word    345
            .word    156
            .word    39
            .word    0
            .word    39
            .word    156
            .word    345
            .word    600
            .word    910
            .word    1264
            .word    1648
;   
;------------------------------------------------------------------------------ 
;           Interrupt Vectors Used MSP430x11x1           
;------------------------------------------------------------------------------ 
            .section    .vectors
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ; 
