#include <io.h>
;******************************************************************************
;   MSP-FET430P120 Demo - ADC10 Sample A10Temp and Convert to oC and oF  
;
;   Description; A single sample is made on A10 with refernce to internal
;   1.5V Vref. Software sets ADC10SC to start sample and conversion - ADC10SC 
;   automatically cleared at EOC. ADC10 internal oscillator/4 times sample 
;   (64x) and conversion. In Mainloop MSP430 waits in LPM0 to save power until 
;   ADC10 conversion complete, ADC10_ISR will force exit from any LPMx in 
;   Mainloop on reti. Result is converted to Temperature represented as 
;   BCD 0000 - 0145 representing oC saved at 0x200 and 0000 - 0292 representing
;   oF saved at 0x202. Temperature sensor offset and slope will vary from device 
;   to device per datasheet tolerence.  
;
;   Uncalibrated temperature measured from device to devive will vary with 
;   slope and offset from device to device - please see datasheet.
//  //*MSP430F1232 or MSP430F1132 Device Required*//
;
;                MSP430F1232
;             -----------------
;         /|\|              XIN|-  
;          | |                 |
;          --|RST          XOUT|-
;            |                 |
;            |A10              |
;
;   M.Buccini
;   Texas Instruments, Inc
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
            .section    .text
RESET:      mov.w   #0x300,R1               ; Initialize stackpointer
StopWDT:    mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop WDT
SetupADC10: mov.w   #INCH_10+ADC10DIV_3,&ADC10CTL1     ; Temp Sensor ADC10CLK/4    
            mov.w   #SREF_1+ADC10SHT_3+REFON+ADC10ON+ADC10IE,&ADC10CTL0 ; 
                                            ;
Mainloop:   bis.w   #ENC+ADC10SC,&ADC10CTL0 ; Start sampling/conversion
            bis.w   #CPUOFF+GIE,R2          ; LPM0, ADC10_ISR will force exit
            call    #Trans2TempC            ; Transform voltage to temperature
            call    #BIN2BCD4               ; R13 = TempC = 0000 - 0145 BCD  
            mov.w   R13,&0x200              ; 0x200 = temperature oC
            call    #Trans2TempF            ; Transform voltage to temperature
            call    #BIN2BCD4               ; R13 = TempF = 0000 - 0292 BCD  
            mov.w   R13,&0x202              ; 0x202 = temperature oF
            jmp     Mainloop                ; << breakpoint here
                                            ;
;------------------------------------------------------------------------------ 
Trans2TempC: ;Subroutine coverts R12 = ADC10MEM/1024*423-278
;           oC = ((x/1024)*1500mV)-986mV)*1/3.55mV = x*423/1024 - 278 
;           Input:  ADC10MEM  0000 - 0FFFh, R11, R12, R14, R15 working register   
;           Output: R12  0x00 - 0x91   
;------------------------------------------------------------------------------ 
            mov.w   &ADC10MEM,R12           ;
            mov.w   #423,R11                ; C          
            call    #MPYU                   ;
            bic.w   #0xFF,R14               ; /1024
            add.w   R15,R14                 ;
            swpb    R14                     ;
            rra.w   R14                     ;
            rra.w   R14                     ;
            mov.w   R14,R12                 ;
            sub.w   #278,R12                ; C    
            ret                             ;
                                            ;
;------------------------------------------------------------------------------ 
Trans2TempF: ;Subroutine coverts R12 = ADC10MEM/1024*761-468
;           oF = ((x/1024)*1500mV)-923mV)*1/1.97mV = x*761/1024 - 468 
;           Input:  ADC10MEM  0000 - 0FFFh, R11, R12, R14, R15 working register   
;           Output: R12  0x000 - 0x124   
;------------------------------------------------------------------------------ 
            mov.w   &ADC10MEM,R12           ;
            mov.w   #761,R11                ; F          
            call    #MPYU                   ;
            bic.w   #0xFF,R14               ; /1024
            add.w   R15,R14                 ;
            swpb    R14                     ;
            rra.w   R14                     ;
            rra.w   R14                     ;
            mov.w   R14,R12                 ;
            sub.w   #468,R12                ; F    
            ret                             ;
                                            ;
;------------------------------------------------------------------------------ 
BIN2BCD4:   ; Subroutine converts binary number R12 -> Packed 4- digit BCD R13
;           Input:  R12  0000 - 0FFFh, R15 working register   
;           Output: R13  0000 - 4095   
;------------------------------------------------------------------------------ 
            mov.w   #16,R15                 ; Loop Counter
            clr.w   R13                     ; 0 -> RESULT LSD
BIN1:       rla.w   R12                     ; Binary MSB to carry
            dadd.w  R13,R13                 ; RESULT x2 LSD
            dec.w   R15                     ; Through?
            jnz     BIN1                    ; Not through
            ret                             ;
                                            ;
;------------------------------------------------------------------------------ 
MPYU:  ;    Unsigned Multipy R11 x R12 = R15|R14
       ;    Input:  R11, R12 -- R10 and R13 are working registers
       ;    Output: R15, R14 
;------------------------------------------------------------------------------ 
            clr.w   R14                     ; 0 -> LSBs result
            clr.w   R15                     ; 0 -> MSBs result
MACU:       clr.w   R13                     ; MSBs multiplier
            mov.w   #1,R10                  ; bit test register
MPY2:       bit.w   R10,R11                 ; test actual bit
            jz      MPY1                    ; IF 0: do nothing
            add.w   R12,R14                 ; IF 1: add multiplier to result
            addc.w  R13,R15                 ;
MPY1:       rla.w   R12                     ; multiplier x 2
            rlc.w   R13                     ;
            rla.w   R10                     ; next bit to test
            jnc     MPY2                    ; if bit in carry: finished
            ret                             ; Return from subroutine
                                            ;
;------------------------------------------------------------------------------ 
ADC10_ISR:
;------------------------------------------------------------------------------ 
            mov.w   #GIE,0(R1)              ; Exit any LPMx on reti
            reti                            ;		 
                                            ;
;------------------------------------------------------------------------------ 
;           Interrupt Vectors Used MSP430x12x2         
;------------------------------------------------------------------------------ 
            .section    .vector
            .org    0x0A,0xFF               ; ADC10 Vector
            .word   ADC10_ISR               ; 
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ; 
