#include <io.h>
;*******************************************************************************
;  MSP-FET430P410 Demo - LCD, displays characters on 14 segment 4 multiplex LCD
;
;  Description "FET410_14segLCD": This program displays up to 6 alphanumeric 
;  characters on a 14 segment LCD display driven directly by the MSP430 MCU.
;  The software reads a character string and displays up to 6 characters on
;  the display.  The MSP430 then waits in low power mode 3 while still
;  maintaining the display.  To use the program chose one of the character
;  strings provided in the software or enter a new one of up to 6 characters.
;  Comment out any strings not used. The character ";" is used for blank spaces.
;  Reset the program then press run. The string will be displayed on the LCD.
;  If the user does not have the LCD connected the functioning of the software 
;  can still be followed by single stepping and watching the register values, 
;  the TEMP value in RAM, and the LCD memory values in the LCD memory locations. 
;   ACLK = LFXT1 = 32768, MCLK = SMCLK = DCO = 32xACLK = 1.048576MHz 
;   //*An external watch crystal is required on XIN/XOUT for ACLK*//	  
;  
;  Connections MSP430F413 -> Varitronix 14 segment LCD part # VIM-828-DP-RC-S-LV
;
;                   TI MSP430F413    LCD        TI MSP430F413    LCD           
;                   ------------- | ---------   ------------- | ---------- 
;                          COM3   |  1                 COM0   |  19
;                          NC     |  2                 NC     |  20
;                          NC     |  3                 NC     |  21  
;                          SEG21  |  4                 SEG0   |  22
;                          SEG23  |  5                 SEG2   |  23 
;                          SEG17  |  6                 SEG4   |  24
;                          SEG19  |  7                 SEG6   |  25
;                          SEG13  |  8                 SEG8   |  26
;                          SEG17  |  9                 SEG10  |  27
;                          SEG9   |  10                SEG12  |  28
;                          SEG11  |  11                SEG14  |  29
;                          SEG5   |  12                SEG16  |  30
;                          SEG7   |  13                SEG18  |  31
;                          SEG1   |  14                SEG20  |  32
;                          SEG3   |  15                SEG22  |  33
;                          NC     |  16                NC     |  34
;                          NC     |  17                NC     |  35
;                          COM2   |  18                COM1   |  36
;
;  NOTE 1: MSP430F413 will drive 6 of 8 14-segment characters so only characters
;          2-7 are connected to center the display. The connections can be 
;          changed to add characters or shift display.
;
;  NOTE 2: Pin R03 on the MSP430 must be connected to GND
;
;   B. Merritt
;   Texas Instruments Inc.
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;
;--------CPU registers used------------------------------------------------------
#define   str_pnt  R5  
                                          ;R5 used as pointer within a string 
#define   tbl_pnt  R6
                                          ;R6 used pointer within the character
                                          ;table 

;--------RAM words used for variables-------------------------------------------
            .equ    TEMP,0x200              ;temporary storage in LCD routine

;-------------------------------------------------------------------------------
            .section    .text
RESET:      mov    #0x300,R1                ;initialize stackpointer          
            call   #Setup                   ;configure MSP430 for application

Mainloop:   mov    #string0,str_pnt         ;addr of 1st byte into string pointer
            call   #Static                  ;call routine for static display
            bis    #LPM3,R2                 ;Set SR bits to wait in LPM3
              
;-------------------------------------------------------------------------------
;         Display string on LCD
;-------------------------------------------------------------------------------
Static:     mov    #10,R15                  ;results in 6 char loaded to LCD mem
Stat_char:  mov.b  0(str_pnt),tbl_pnt       ;char pointed to, into table pointer
            add.b  0(str_pnt),tbl_pnt       ;value x 2 since is 2 bytes in table
            tst    tbl_pnt                  ;test value
            jz     Staticend                ;if 0 (= end of string) then jump 
            mov    LCD_tbl-86(tbl_pnt),TEMP ;char word from table to temp location
                                            ;-86 adjusts from 2 x ASCII value 
            mov.b  TEMP,LCDM1(R15)          ;low byte of char to LCD current digit
            swpb   TEMP                     ;swap upper and lower bytes in temp
            mov.b  TEMP,LCDM2(R15)          ;high byte of char to LCD digit 
            inc    str_pnt                  ;increment string pointer to next byte
            decd   R15                      ;double decrement charater index
            jmp    Stat_char                ;loop to get & load next char
Staticend:  ret

;-------------------------------------------------------------------------------
Setup:    ;Configure modules and control registers 
;-------------------------------------------------------------------------------
StopWDT:  mov    #WDTPW+WDTHOLD,&WDTCTL    ;stop Watchdog Timer
SetupFLL2: bis.b  #XCAP18PF,&FLL_CTL0       ;set load capacitance for xtal
          mov    #10000,R15
FLL_Wait: dec    R15                       ;delay for FLL to lock
          jnz    FLL_Wait	

SetupLCD: mov.b  #0x7D,&LCDCTL             ;4mux LCD, segs = 0-23
          bis.b  #0xFC,&P5SEL              ;set Rxx and COM pins for LCD
SetupBT:  mov.b  #0x00+BTFRFQ1,&BTCTL      ;set fLCD = ACLK / 128
SetPorts: mov.b  #0xFF,&P1DIR              ;set port to outputs
SetupPx:  mov.b  #0xFF,&P2DIR              ;set port to outputs
          mov.b  #0xFF,&P3DIR              ;set port to outputs
          mov.b  #0xFF,&P4DIR              ;set port to outputs
          mov.b  #0xFF,&P6DIR              ;set port to outputs

ClearLCD: mov    #15,R15                   ;15 LCD memory bytes to clear
Clear1:   mov.b  #0,LCDM1(R15)             ;write zeros in LCD RAM locations
                                           ;so clear display
          dec    R15                       ;all LCD mem clear?
          jc     Clear1                    ;more LCD mem to clear go, use JC
                                           ;to get memory location 0
          mov.b  #LCDM1,R15;               ;R15 points to first LCD location
          ret

;-------------------------------------------------------------------------------
;         Define character string to display.  The ";" character = blank space
;-------------------------------------------------------------------------------
string0:    .byte   'M','S','P','4','3','0',0

;-------------------------------------------------------------------------------
;        The following table defines the LCD segements to be displayed for each 
;        character.  Software reads the ASCII value of the character desired
;        then adjusts the offset to find the correct table member. The 
;        punctuation characters (ASCII 0x39-0x40) are used to define other desired
;        characters. Use the corresponding ASCII character to get the desired 
;        character on the display.
;-------------------------------------------------------------------------------
            .align  1
LCD_tbl:    .word   0x5A00     ;displays +                            ASCII hex=0x2B=43 
            .word   0xAAAA     ;undefined display is meaningless
            .word   0x4200     ;displays -
            .word   0x0008     ;displays . decimal point not period
            .word   0x2400     ;displays /
            ;*********************************************************************          
            .word   0x24E7     ;displays 0                            ASCII hex=0x30=48
            .word   0x0006     ;displays 1
            .word   0x42C3     ;displays 2
            .word   0x4287     ;displays 3
            .word   0x4226     ;displays 4
            .word   0x42A5     ;displays 5          
            .word   0x42E5     ;displays 6
            .word   0x0007     ;displays 7
            .word   0x42E7     ;displays 8
            .word   0x42A7     ;displays 9
            ;*********************************************************************
            .word   0x1800     ;displays ":" vertical bars(ASCII ":") ASCII hex=0x39=58 
            .word   0x0000     ;displays " " blank space (ASCII ";")
            .word   0x00C4     ;displays "u" used as the micro symbol 'mu' (ASCII "<")
            .word   0xAAAA     ;undefined display is meaningless
            .word   0xAAAA     ;undefined display is meaningless
            .word   0xAAAA     ;undefined display is meaningless
            .word   0x42A7     ;displays "o" symbol for temperature (ASCII "@")
            ;*********************************************************************
            .word   0x4267     ;displays A                            ASCII hex=0x41=65
            .word   0x5887     ;displays B
            .word   0x00E1     ;displays C
            .word   0x1887     ;displays D
            .word   0x42E1     ;displays E
            .word   0x4261     ;displays F
            .word   0x40E5     ;displays G
            .word   0x4266     ;displays H
            .word   0x1881     ;displays I
            .word   0x00C6     ;displays J          
            .word   0xA260     ;displays K
            .word   0x00E0     ;displays L
            .word   0x2166     ;displays M
            .word   0x8166     ;displays N
            .word   0x00E7     ;displays O
            .word   0x4263     ;displays P
            .word   0x80E7     ;displays Q
            .word   0xC263     ;displays R
            .word   0x42A5     ;displays S
            .word   0x1801     ;displays T
            .word   0x00E6     ;displays U
            .word   0x2460     ;displays V
            .word   0x8466     ;displays W
            .word   0xA500     ;displays X
            .word   0x4A22     ;displays Y
            .word   0x2481     ;displays Z                            ASCII hex=5Ah=90
          
;-------------------------------------------------------------------------------
;        41x Interrupt vectors
;------------------------------------------------------------------------------- 
            .section    .vectors
            .org    0x1E,0xFF           ;RESET Vector
            .word   RESET
